package command

import (
	"gitlab.schukai.com/oss/utilities/conan/configuration"
	"gitlab.schukai.com/oss/utilities/conan/logging"
	"gitlab.schukai.com/oss/utilities/conan/server"
	"os"
	"path"
)

type ServerServeCommand struct {
	Port      string   `long:"port" short:"p" description:"Port to listen on"`
	Address   string   `long:"address" short:"a" description:"Address to serve on"`
	WebPath   string   `long:"web-path" short:"e" description:"Path to web files"`
	WatchPath string   `long:"watch-path" short:"w" description:"Path to watch for changes"`
	Exclude   []string `long:"exclude" short:"x" description:"Exclude pattern"`
}

func (r *ServerServeCommand) execute() {

	logging.InitLogger()

	configuration.SetServerPort(r.Port)
	configuration.SetServerAddress(r.Address)
	configuration.SetServerWebPath(r.WebPath)

	var w string
	if r.WatchPath != "" {
		w = r.WatchPath
		if !path.IsAbs(w) {
			wd, err := os.Getwd()
			if err != nil {
				logging.LogError("ServerServeCommand error %s", err.Error())
				return
			}
			w = path.Join(wd, w)
		}

		configuration.AddWatch(configuration.Watch{
			Path:    w,
			Exclude: r.Exclude,
		})
	}

	server.Start()

}
