package command

import (
	"github.com/jessevdk/go-flags"
	"gitlab.schukai.com/oss/utilities/conan/configuration"
	error2 "gitlab.schukai.com/oss/utilities/conan/error"
	"gitlab.schukai.com/oss/utilities/conan/release"
)

func Execute() {

	cmdInst.parser = flags.NewParser(cmdInst.definition, flags.Default)

	_, err := cmdInst.parser.Parse()
	if err != nil {
		return
	}

	configuration.InitConfiguration(cmdInst.definition.ConfigPath, release.GetMnemonic())

	c := cmdInst.parser.Command
	if c == nil || c.Active == nil {
		error2.PrintErrorAndExit("No command specified")
	}

	name := getActiveCommandName(c.Active, "")
	if name == "" {
		error2.PrintErrorAndExit("Unknown command: " + c.Name)
	}

	p, ok := cmdInst.handler[name]
	if !ok {
		error2.PrintErrorAndExit("No handler for command: " + name)
	}

	p.execute()

}

func getActiveCommandName(c *flags.Command, base string) string {
	if c == nil {
		return base
	}

	if c.Active != nil {
		return getActiveCommandName(c.Active, c.Name)
	}

	if base != "" {
		base = base + "-"
	}

	return base + c.Name

}
