package configuration

import (
	"context"
	envconfig "github.com/sethvargo/go-envconfig"
	"gitlab.schukai.com/oss/utilities/conan/constants"
	error2 "gitlab.schukai.com/oss/utilities/conan/error"
	"gitlab.schukai.com/oss/utilities/conan/logging"
	"gopkg.in/yaml.v3"
	"os"
	"os/user"
	"path"
	"strings"
)

var configurationInstance *ConfigStruct
var configurationPath string

func init() {
	configurationInstance = &ConfigStruct{}
}

func InitConfiguration(cfgPath, mnemonic string) {

	userConfig := ""

	usr, err := user.Current()
	if err == nil {
		userConfig = path.Join(usr.HomeDir, ".config", mnemonic, constants.ConfigFileName)
	}

	var currentConfigPath string

	current, err := os.Getwd()
	if err == nil {
		currentConfigPath = path.Join(current, constants.ConfigFileName)
	}

	etcConfigPath := path.Join("/etc", mnemonic, constants.ConfigFileName)

	if cfgPath != "" {

		if !path.IsAbs(cfgPath) {
			cfgPath = path.Join(current, cfgPath)
		}

		if _, err := os.Stat(cfgPath); os.IsNotExist(err) {
			error2.PrintErrorAndExit("Configuration file not found: " + cfgPath)
		}

	}
	
	ctx := context.Background()

	l := envconfig.PrefixLookuper(strings.ReplaceAll(strings.ToUpper(mnemonic), "-", "_")+"_", envconfig.OsLookuper())
	err = envconfig.ProcessWith(ctx, configurationInstance, l)

	error2.CheckError(err)

	for _, p := range []string{
		cfgPath,
		currentConfigPath,
		userConfig,
		etcConfigPath} {
		if checkAndInitConfiguration(p, configurationInstance) {
			configurationPath = path.Dir(p)
			break
		}
	}

	sanitize()

}

func Reload() {
	logging.LogInfo("Reloading configuration")
	checkAndInitConfiguration(path.Join(configurationPath, constants.ConfigFileName), configurationInstance)
}

func checkAndInitConfiguration(name string, config *ConfigStruct) bool {

	if name == "" {
		return false
	}

	if _, err := os.Stat(name); os.IsNotExist(err) {
		return false
	}

	ptr, err := os.Open(name)
	if err != nil {
		return false
	}
	defer ptr.Close()

	decoder := yaml.NewDecoder(ptr)
	err = decoder.Decode(config)

	if err != nil {
		return false
	}

	return true
}
